#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from esi_leap.tests.api import base as test_api_base


class FakeIronicNode(object):
    def __init__(self):
        self.name = 'fake-node'
        self.owner = 'fake-project-uuid'
        self.uuid = 'fake-uuid'
        self.properties = {'lease_uuid': 'fake-lease-uuid'}
        self.lessee = 'fake-project-uuid'
        self.maintenance = False
        self.provision_state = 'active'


class FakeProject(object):
    def __init__(self):
        self.name = 'fake-project'
        self.id = 'fake-project-uuid'


class TestNodesController(test_api_base.APITestCase):

    def setUp(self):
        super(TestNodesController, self).setUp()

    @mock.patch('esi_leap.common.ironic.get_node_list')
    @mock.patch('esi_leap.objects.offer.Offer.get_all')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    @mock.patch('esi_leap.common.keystone.get_project_list')
    def test_get_all(self, mock_gpl, mock_lga, mock_oga, mock_gnl):
        fake_node = FakeIronicNode()
        fake_project = FakeProject()
        mock_gnl.return_value = [fake_node]
        mock_oga.return_value = []
        mock_lga.return_value = []
        mock_gpl.return_value = [fake_project]

        data = self.get_json('/nodes')

        mock_gnl.assert_called_once_with(self.context)
        mock_oga.assert_called_once()
        mock_lga.assert_called_once()
        mock_gpl.assert_called_once()

        self.assertEqual(data['nodes'][0]['name'], 'fake-node')
        self.assertEqual(data['nodes'][0]['uuid'], 'fake-uuid')
        self.assertEqual(data['nodes'][0]['owner'], 'fake-project')
        self.assertEqual(data['nodes'][0]['lease_uuid'], 'fake-lease-uuid')
        self.assertEqual(data['nodes'][0]['lessee'], 'fake-project')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

UNKNOWN = {
    'uuid': 'unknown-uuid',
    'name': 'unknown-name',
    'resource_class': 'unknown-class',
    'config': {},
    'owner_project_id': 'unknown-owner',
    'lease_uuid': 'unknown-lease',
    'lessee_project_id': 'unknown-lessee',
    'provision_state': 'unknown-provision-state',
    'power_state': 'unknown-power-state'
}



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import mock
from oslo_context import context as ctx
from oslo_policy import policy as oslo_policy
from oslo_utils import uuidutils

import testtools

from esi_leap.api.controllers.v1 import utils
from esi_leap.common import exception
from esi_leap.common import policy
from esi_leap.common import statuses
from esi_leap.objects import lease
from esi_leap.objects import offer
from esi_leap.resource_objects.test_node import TestNode


admin_ctx = ctx.RequestContext(project_id='adminid', roles=['admin'])

owner_ctx = ctx.RequestContext(project_id='ownerid', roles=['owner'])
owner_ctx_dict = owner_ctx.to_policy_values()

lessee_ctx = ctx.RequestContext(project_id='lesseeid', roles=['lessee'])
lessee_ctx_dict = lessee_ctx.to_policy_values()

owner_ctx_2 = ctx.RequestContext(project_id='ownerid2', roles=['owner'])
lessee_ctx_2 = ctx.RequestContext(project_id='lesseeid2', roles=['lessee'])


start = datetime.datetime(2016, 7, 16)
start_iso = '2016-07-16T00:00:00'

end = start + datetime.timedelta(days=100)
end_iso = '2016-10-24T00:00:00'

test_node_1 = TestNode('111', owner_ctx.project_id)
test_node_2 = TestNode('bbb', owner_ctx_2.project_id)

o_uuid = uuidutils.generate_uuid()


test_offer = offer.Offer(
    resource_type='test_node',
    resource_uuid=test_node_1._uuid,
    name='o',
    uuid=o_uuid,
    lessee_id=None,
    status=statuses.AVAILABLE,
    start_time=start,
    end_time=end,
    project_id=owner_ctx.project_id
)

test_offer_2 = offer.Offer(
    resource_type='test_node',
    resource_uuid=test_node_1._uuid,
    name='o',
    uuid=uuidutils.generate_uuid(),
    lessee_id=None,
    status=statuses.EXPIRED,
    start_time=start,
    end_time=end,
    project_id=owner_ctx.project_id
)

test_offer_lessee_match = offer.Offer(
    resource_type='test_node',
    resource_uuid=test_node_1._uuid,
    name='o',
    uuid=uuidutils.generate_uuid(),
    lessee_id='lesseeid',
    status=statuses.EXPIRED,
    start_time=start,
    end_time=end,
    project_id=owner_ctx.project_id
)

test_offer_lessee_no_match = offer.Offer(
    resource_type='test_node',
    resource_uuid=test_node_1._uuid,
    name='o',
    uuid=uuidutils.generate_uuid(),
    lessee_id='otherlesseeid',
    status=statuses.EXPIRED,
    start_time=start,
    end_time=end,
    project_id=owner_ctx.project_id
)

test_lease = lease.Lease(
    offer_uuid=o_uuid,
    name='c',
    uuid=uuidutils.generate_uuid(),
    project_id=lessee_ctx.project_id,
    owner_id=owner_ctx.project_id,
    status=statuses.CREATED
)

test_lease_2 = lease.Lease(
    offer_uuid=o_uuid,
    name='c',
    uuid=uuidutils.generate_uuid(),
    project_id=lessee_ctx.project_id,
    owner_id=owner_ctx.project_id,
    status=statuses.CREATED
)

test_lease_3 = lease.Lease(
    offer_uuid=o_uuid,
    name='c',
    uuid=uuidutils.generate_uuid(),
    project_id=lessee_ctx_2.project_id,
    owner_id=owner_ctx.project_id,
    status=statuses.CREATED
)

test_lease_4 = lease.Lease(
    offer_uuid=o_uuid,
    name='c2',
    uuid=uuidutils.generate_uuid(),
    project_id=lessee_ctx_2.project_id,
    owner_id=owner_ctx.project_id,
    status=statuses.CREATED
)


class TestGetObjectUtils(testtools.TestCase):

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.offer.Offer.get')
    def test_get_offer_uuid(self, mock_offer_get, mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_offer_get.return_value = test_offer

        res = utils.get_offer(test_offer.uuid)

        self.assertEqual(res, test_offer)

        mock_is_uuid_like.assert_called_once_with(test_offer.uuid)
        mock_offer_get.assert_called_once_with(test_offer.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.offer.Offer.get')
    def test_get_offer_uuid_available(self, mock_offer_get, mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_offer_get.return_value = test_offer

        res = utils.get_offer(test_offer.uuid, statuses.AVAILABLE)

        self.assertEqual(res, test_offer)

        mock_is_uuid_like.assert_called_once_with(test_offer.uuid)
        mock_offer_get.assert_called_once_with(test_offer.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.offer.Offer.get')
    def test_get_offer_uuid_bad_status(self, mock_offer_get,
                                       mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_offer_get.return_value = test_offer

        self.assertRaises(exception.OfferNotFound,
                          utils.get_offer,
                          test_offer.uuid, statuses.DELETED)

        mock_is_uuid_like.assert_called_once_with(test_offer.uuid)
        mock_offer_get.assert_called_once_with(test_offer.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.offer.Offer.get_all')
    def test_get_offer_name_available(self, mock_offer_get_all,
                                      mock_is_uuid_like):

        mock_is_uuid_like.return_value = False
        mock_offer_get_all.return_value = [test_offer]

        res = utils.get_offer(test_offer.name, statuses.AVAILABLE)

        self.assertEqual(res, test_offer)

        mock_is_uuid_like.assert_called_once_with(test_offer.name)
        mock_offer_get_all.assert_called_once_with(
            {'name': test_offer.name,
             'status': statuses.AVAILABLE})

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.lease.Lease.get')
    def test_get_lease_uuid(self, mock_lease_get, mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_lease_get.return_value = test_lease

        res = utils.get_lease(test_lease.uuid)

        self.assertEqual(res, test_lease)

        mock_is_uuid_like.assert_called_once_with(test_lease.uuid)
        mock_lease_get.assert_called_once_with(test_lease.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.lease.Lease.get')
    def test_get_lease_uuid_available(self, mock_lease_get, mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_lease_get.return_value = test_lease

        res = utils.get_lease(test_lease.uuid, [statuses.CREATED])

        self.assertEqual(res, test_lease)

        mock_is_uuid_like.assert_called_once_with(test_lease.uuid)
        mock_lease_get.assert_called_once_with(test_lease.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.lease.Lease.get')
    def test_get_lease_uuid_bad_status(self, mock_lease_get,
                                       mock_is_uuid_like):

        mock_is_uuid_like.return_value = True
        mock_lease_get.return_value = test_lease

        self.assertRaises(exception.LeaseNotFound,
                          utils.get_lease,
                          test_lease.uuid, statuses.DELETED)

        mock_is_uuid_like.assert_called_once_with(test_lease.uuid)
        mock_lease_get.assert_called_once_with(test_lease.uuid)

    @mock.patch('oslo_utils.uuidutils.is_uuid_like')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test_get_lease_name_active(self, mock_lease_get_all,
                                   mock_is_uuid_like):

        mock_is_uuid_like.return_value = False
        mock_lease_get_all.return_value = [test_lease]

        res = utils.get_lease(test_lease.name, statuses.ACTIVE)

        self.assertEqual(res, test_lease)

        mock_is_uuid_like.assert_called_once_with(test_lease.name)
        mock_lease_get_all.assert_called_once_with(
            {'name': test_lease.name,
             'status': statuses.ACTIVE})


class TestCheckResourceAdminUtils(testtools.TestCase):

    @mock.patch.object(test_node_1, 'get_owner_project_id')
    @mock.patch('esi_leap.api.controllers.v1.utils.resource_policy_authorize')
    def test_check_resource_admin_owner(self, mock_authorize, mock_gopi):
        mock_gopi.return_value = owner_ctx.project_id

        utils.check_resource_admin(owner_ctx.to_policy_values(),
                                   test_node_1,
                                   test_offer.project_id)
        mock_gopi.assert_called_once()
        mock_authorize.assert_not_called()

    @mock.patch.object(test_node_2, 'get_owner_project_id')
    @mock.patch('esi_leap.api.controllers.v1.utils.resource_policy_authorize')
    def test_check_resource_admin_admin(self, mock_authorize, mock_gopi):
        mock_gopi.return_value = owner_ctx_2.project_id

        bad_test_offer = offer.Offer(
            resource_type='test_node',
            resource_uuid=test_node_2._uuid,
            project_id=owner_ctx.project_id
        )

        utils.check_resource_admin(admin_ctx.to_policy_values(),
                                   test_node_2,
                                   bad_test_offer.project_id)

        mock_gopi.assert_called_once()
        mock_authorize.assert_called_once_with('esi_leap:offer:offer_admin',
                                               admin_ctx.to_policy_values(),
                                               admin_ctx.to_policy_values(),
                                               'test_node', test_node_2._uuid)

    @mock.patch.object(test_node_2, 'get_owner_project_id')
    @mock.patch('esi_leap.api.controllers.v1.utils.resource_policy_authorize')
    def test_check_resource_admin_invalid_owner(self, mock_authorize,
                                                mock_gopi):
        mock_gopi.return_value = owner_ctx_2.project_id
        mock_authorize.side_effect = exception.HTTPResourceForbidden(
            resource_type='test_node', resource=test_node_2._uuid)

        bad_test_offer = offer.Offer(
            resource_type='test_node',
            resource_uuid=test_node_2._uuid,
            project_id=owner_ctx.project_id
        )

        self.assertRaises(exception.HTTPResourceForbidden,
                          utils.check_resource_admin,
                          owner_ctx_2.to_policy_values(),
                          test_node_2,
                          bad_test_offer.project_id)

        mock_gopi.assert_called_once()
        mock_authorize.assert_called_once_with('esi_leap:offer:offer_admin',
                                               owner_ctx_2.to_policy_values(),
                                               owner_ctx_2.to_policy_values(),
                                               'test_node', test_node_2._uuid)


class TestCheckResourceLeaseAdminUtils(testtools.TestCase):

    def setUp(self):
        super(TestCheckResourceLeaseAdminUtils, self).setUp()

        self.test_lease = lease.Lease(
            uuid=uuidutils.generate_uuid(),
            start_time=datetime.datetime(2016, 7, 16, 19, 20, 30),
            end_time=datetime.datetime(2016, 8, 16, 19, 20, 30),
            parent_lease_uuid=None
        )
        self.test_lease_with_parent = lease.Lease(
            uuid=uuidutils.generate_uuid(),
            start_time=datetime.datetime(2016, 7, 16, 19, 20, 30),
            end_time=datetime.datetime(2016, 8, 16, 19, 20, 30),
            parent_lease_uuid='parent-lease-uuid'
        )

    @mock.patch('esi_leap.objects.lease.Lease.get')
    @mock.patch.object(test_node_1, 'get_lease_uuid',
                       return_value='a-lease-uuid')
    @mock.patch.object(test_node_1, 'get_lessee_project_id',
                       return_value=test_offer.project_id)
    def test_check_resource_lease_admin_okay(self, mock_glpi, mock_glu,
                                             mock_lease_get):
        mock_lease_get.return_value = self.test_lease
        parent_lease_uuid = utils.check_resource_lease_admin(
            owner_ctx.to_policy_values(),
            test_node_1,
            test_offer.project_id,
            datetime.datetime(2016, 7, 16, 19, 20, 30),
            datetime.datetime(2016, 8, 16, 19, 20, 30))

        mock_glpi.assert_called_once()
        mock_glu.assert_called_once()
        mock_lease_get.assert_called_once_with('a-lease-uuid')
        self.assertEqual('a-lease-uuid', parent_lease_uuid)

    @mock.patch('esi_leap.objects.lease.Lease.get')
    @mock.patch.object(test_node_1, 'get_lease_uuid',
                       return_value='a-lease-uuid')
    @mock.patch.object(test_node_1, 'get_lessee_project_id',
                       return_value='other-lessee')
    def test_check_resource_lease_admin_not_lessee(self, mock_glpi, mock_glu,
                                                   mock_lease_get):
        mock_lease_get.return_value = self.test_lease
        parent_lease_uuid = utils.check_resource_lease_admin(
            owner_ctx.to_policy_values(),
            test_node_1,
            test_offer.project_id,
            datetime.datetime(2016, 7, 16, 19, 20, 30),
            datetime.datetime(2016, 8, 16, 19, 20, 30))

        mock_glpi.assert_called_once()
        mock_glu.assert_not_called()
        mock_lease_get.assert_not_called()
        self.assertIsNone(parent_lease_uuid)

    @mock.patch('esi_leap.objects.lease.Lease.get')
    @mock.patch.object(test_node_1, 'get_lease_uuid',
                       return_value=None)
    @mock.patch.object(test_node_1, 'get_lessee_project_id',
                       return_value=test_offer.project_id)
    def test_check_resource_lease_admin_not_lease(self,
                                                  mock_glpi,
                                                  mock_glu,
                                                  mock_lease_get):
        mock_lease_get.return_value = self.test_lease
        parent_lease_uuid = utils.check_resource_lease_admin(
            owner_ctx.to_policy_values(),
            test_node_1,
            test_offer.project_id,
            datetime.datetime(2016, 7, 16, 19, 20, 30),
            datetime.datetime(2016, 8, 16, 19, 20, 30))

        mock_glpi.assert_called_once()
        mock_glu.assert_called_once()
        mock_lease_get.assert_not_called()
        self.assertIsNone(parent_lease_uuid)

    @mock.patch('esi_leap.objects.lease.Lease.get')
    @mock.patch.object(test_node_1, 'get_lease_uuid',
                       return_value='a-lease-uuid')
    @mock.patch.object(test_node_1, 'get_lessee_project_id',
                       return_value=test_offer.project_id)
    def test_check_resource_lease_admin_parent_lease(self,
                                                     mock_glpi,
                                                     mock_glu,
                                                     mock_lease_get):
        mock_lease_get.return_value = self.test_lease_with_parent
        parent_lease_uuid = utils.check_resource_lease_admin(
            owner_ctx.to_policy_values(),
            test_node_1,
            test_offer.project_id,
            datetime.datetime(2016, 7, 16, 19, 20, 30),
            datetime.datetime(2016, 8, 16, 19, 20, 30))

        mock_glpi.assert_called_once()
        mock_glu.assert_called_once()
        mock_lease_get.assert_called_once_with('a-lease-uuid')
        self.assertIsNone(parent_lease_uuid)

    @mock.patch('esi_leap.objects.lease.Lease.get')
    @mock.patch.object(test_node_1, 'get_lease_uuid',
                       return_value='a-lease-uuid')
    @mock.patch.object(test_node_1, 'get_lessee_project_id',
                       return_value=test_offer.project_id)
    def test_check_resource_lease_admin_bad_time(self, mock_glpi, mock_glu,
                                                 mock_lease_get):
        mock_lease_get.return_value = self.test_lease
        self.assertRaises(exception.ResourceNoPermissionTime,
                          utils.check_resource_lease_admin,
                          owner_ctx.to_policy_values(),
                          test_node_1,
                          test_offer.project_id,
                          datetime.datetime(2016, 7, 15, 19, 20, 30),
                          datetime.datetime(2016, 8, 16, 19, 20, 30))
        mock_glpi.assert_called_once()
        mock_glu.assert_called_once()
        mock_lease_get.assert_called_once_with('a-lease-uuid')


class TestOfferPolicyAndRetrieveUtils(testtools.TestCase):

    @mock.patch('esi_leap.api.controllers.v1.utils.resource_policy_authorize')
    @mock.patch('esi_leap.api.controllers.v1.utils.get_offer')
    def test_check_offer_policy(self, mock_get_offer, mock_authorize):
        mock_get_offer.return_value = test_offer
        target = dict(admin_ctx.to_policy_values())
        target['offer.project_id'] = test_offer.project_id

        offer = utils.check_offer_policy_and_retrieve(
            admin_ctx, 'test_policy:test', '12345', [])

        mock_authorize.assert_called_with('test_policy:test',
                                          target,
                                          admin_ctx.to_policy_values(),
                                          'offer',
                                          test_offer.uuid)
        mock_get_offer.assert_called_with('12345', [])
        self.assertEqual(test_offer, offer)


class TestLeasePolicyAndRetrieveUtils(testtools.TestCase):

    @mock.patch('esi_leap.api.controllers.v1.utils.resource_policy_authorize')
    @mock.patch('esi_leap.api.controllers.v1.utils.get_lease')
    def test_check_lease_policy(self, mock_get_lease, mock_authorize):
        mock_get_lease.return_value = test_lease
        target = dict(admin_ctx.to_policy_values())
        target['lease.owner_id'] = test_lease.owner_id
        target['lease.project_id'] = test_lease.project_id

        lease = utils.check_lease_policy_and_retrieve(
            admin_ctx, 'test_policy:test', '12345', [])

        mock_authorize.assert_called_with('test_policy:test',
                                          target,
                                          admin_ctx.to_policy_values(),
                                          'lease',
                                          test_lease.uuid)
        mock_get_lease.assert_called_with('12345', [])
        self.assertEqual(test_lease, lease)


class TestOfferLesseeUtils(testtools.TestCase):

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_parent_project_id_tree')
    def test_check_offer_lessee_no_lessee_id(self,
                                             mock_gppit,
                                             mock_authorize):
        utils.check_offer_lessee(lessee_ctx.to_policy_values(),
                                 test_offer)

        assert not mock_authorize.called
        assert not mock_gppit.called

    @mock.patch.object(policy, 'authorize', spec=True)
    @mock.patch('esi_leap.common.keystone.get_parent_project_id_tree')
    def test_check_offer_lessee_owner_match(self,
                                            mock_gppit,
                                            mock_authorize):
        utils.check_offer_lessee(owner_ctx.to_policy_values(),
                                 test_offer_lessee_no_match)

        assert not mock_authorize.called
        assert not mock_gppit.called

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_parent_project_id_tree')
    def test_check_offer_lessee_admin(self,
                                      mock_gppit,
                                      mock_authorize):
        mock_authorize.return_value = True
        mock_gppit.return_value = [admin_ctx.project_id]

        utils.check_offer_lessee(admin_ctx.to_policy_values(),
                                 test_offer_lessee_no_match)

        mock_authorize.assert_called_once_with('esi_leap:offer:offer_admin',
                                               admin_ctx.to_policy_values(),
                                               admin_ctx.to_policy_values())
        mock_gppit.assert_called_once_with(admin_ctx.project_id)

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_parent_project_id_tree')
    def test_check_offer_lessee_non_admin_match(self,
                                                mock_gppit,
                                                mock_authorize):
        mock_gppit.return_value = [lessee_ctx.project_id, 'lesseeidparent']

        utils.check_offer_lessee(lessee_ctx.to_policy_values(),
                                 test_offer_lessee_match)

        assert not mock_authorize.called
        mock_gppit.assert_called_once_with(lessee_ctx.project_id)

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_parent_project_id_tree')
    def test_check_offer_lessee_non_admin_no_match(self,
                                                   mock_gppit,
                                                   mock_authorize):
        mock_authorize.side_effect = exception.HTTPResourceForbidden(
            resource_type='offer', resource=test_offer_lessee_no_match.uuid)
        mock_gppit.return_value = [lessee_ctx.project_id, 'lesseeidparent']

        self.assertRaises(exception.HTTPResourceForbidden,
                          utils.check_offer_lessee,
                          lessee_ctx.to_policy_values(),
                          test_offer_lessee_no_match)

        mock_authorize.assert_called_once_with('esi_leap:offer:offer_admin',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())
        mock_gppit.assert_called_once_with(lessee_ctx.project_id)


class TestPolicyAuthorizeUtils(testtools.TestCase):

    @mock.patch.object(policy, 'authorize', spec=True)
    def test_policy_authorize(self, mock_authorize):
        utils.policy_authorize('test_policy:test',
                               lessee_ctx.to_policy_values(),
                               lessee_ctx.to_policy_values())

        mock_authorize.assert_called_once_with('test_policy:test',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())

    @mock.patch.object(policy, 'authorize', spec=True)
    def test_policy_authorize_exception(self, mock_authorize):
        mock_authorize.side_effect = oslo_policy.PolicyNotAuthorized(
            'esi_leap:offer:offer_admin',
            lessee_ctx.to_dict(), lessee_ctx.to_dict())

        self.assertRaises(exception.HTTPForbidden,
                          utils.policy_authorize,
                          'test_policy:test',
                          lessee_ctx.to_policy_values(),
                          lessee_ctx.to_policy_values())

        mock_authorize.assert_called_once_with('test_policy:test',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    def test_resource_policy_authorize(self, mock_authorize):
        utils.resource_policy_authorize('test_policy:test',
                                        lessee_ctx.to_policy_values(),
                                        lessee_ctx.to_policy_values(),
                                        'test_node',
                                        '12345')

        mock_authorize.assert_called_once_with('test_policy:test',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    def test_resource_policy_authorize_exception(self, mock_authorize):
        mock_authorize.side_effect = exception.HTTPForbidden(
            'test_policy:test')

        self.assertRaises(exception.HTTPResourceForbidden,
                          utils.resource_policy_authorize,
                          'test_policy:test',
                          lessee_ctx.to_policy_values(),
                          lessee_ctx.to_policy_values(),
                          'test_node', '12345')
        mock_authorize.assert_called_once_with('test_policy:test',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())


class TestOfferGetDictWithAddedInfoUtils(testtools.TestCase):

    @mock.patch('esi_leap.common.keystone.get_project_name')
    @mock.patch('esi_leap.objects.offer.Offer.get_availabilities')
    def test_offer_get_dict_with_added_info(self,
                                            mock_get_availabilities,
                                            mock_gpn):
        mock_get_availabilities.return_value = []
        mock_gpn.return_value = 'project-name'

        start = datetime.datetime(2016, 7, 16)
        o = offer.Offer(
            resource_type='test_node',
            resource_uuid='1234567890',
            name='o',
            status=statuses.AVAILABLE,
            start_time=start,
            end_time=start + datetime.timedelta(days=100),
            project_id=uuidutils.generate_uuid(),
            lessee_id=None
        )

        o_dict = utils.offer_get_dict_with_added_info(o)

        expected_offer_dict = {
            'resource_type': o.resource_type,
            'resource_uuid': o.resource_uuid,
            'resource_class': 'fake',
            'resource': 'test-node-1234567890',
            'name': o.name,
            'project_id': o.project_id,
            'project': 'project-name',
            'lessee_id': None,
            'lessee': 'project-name',
            'start_time': o.start_time,
            'end_time': o.end_time,
            'status': o.status,
            'availabilities': [],
        }

        self.assertEqual(expected_offer_dict, o_dict)
        self.assertEqual(2, mock_gpn.call_count)


class TestLeaseGetDictWithAddedInfoUtils(testtools.TestCase):

    def setUp(self):
        super(TestLeaseGetDictWithAddedInfoUtils, self).setUp()

        self.test_lease = lease.Lease(
            start_time=datetime.datetime(2016, 7, 16, 19, 20, 30),
            end_time=datetime.datetime(2016, 8, 16, 19, 20, 30),
            uuid=uuidutils.generate_uuid(),
            resource_type='test_node',
            resource_uuid='111',
            project_id='lesseeid',
            owner_id='ownerid',
            parent_lease_uuid=None
        )

    @mock.patch('esi_leap.resource_objects.test_node.TestNode.get_name')
    @mock.patch('esi_leap.common.keystone.get_project_name')
    @mock.patch('esi_leap.objects.lease.get_resource_object')
    def test_lease_get_dict_with_added_info(self, mock_gro, mock_gpn, mock_gn):
        mock_gro.return_value = TestNode('111')
        mock_gpn.return_value = 'project-name'
        mock_gn.return_value = 'resource-name'

        output_dict = utils.lease_get_dict_with_added_info(self.test_lease)

        expected_output_dict = self.test_lease.to_dict()
        expected_output_dict['resource'] = 'resource-name'
        expected_output_dict['project'] = 'project-name'
        expected_output_dict['owner'] = 'project-name'
        expected_output_dict['resource_class'] = 'fake'

        mock_gro.assert_called_once()
        self.assertEqual(2, mock_gpn.call_count)
        mock_gn.assert_called_once()
        self.assertEqual(expected_output_dict, output_dict)


class TestCheckLeaseLength(testtools.TestCase):
    def setUp(self):
        super(TestCheckLeaseLength, self).setUp()
        self.max_time = 21
        self.start_time = datetime.datetime(2016, 7, 16, 19, 20, 30)
        self.end_time = datetime.datetime(2016, 8, 16, 19, 20, 30)
        self.end_time_1 = datetime.datetime(2016, 7, 20, 19, 20, 30)

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    def test_check_lease_length_admin_exceed_max(self, mock_authorize):
        utils.check_lease_length(admin_ctx.to_policy_values(),
                                 self.start_time,
                                 self.end_time,
                                 self.max_time)

        mock_authorize.assert_called_once_with('esi_leap:lease:lease_admin',
                                               admin_ctx.to_policy_values(),
                                               admin_ctx.to_policy_values())

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    def test_check_lease_length_exception(self, mock_authorize):
        mock_authorize.side_effect = exception.HTTPForbidden(
            'esi_leap:lease:lease_admin')

        self.assertRaises(exception.LeaseExceedMaxTimeRange,
                          utils.check_lease_length,
                          lessee_ctx.to_policy_values(),
                          self.start_time,
                          self.end_time,
                          self.max_time)

        mock_authorize.assert_called_once_with('esi_leap:lease:lease_admin',
                                               lessee_ctx.to_policy_values(),
                                               lessee_ctx.to_policy_values())

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    def test_check_lease_length_non_admin_valid(self, mock_authorize):
        utils.check_lease_length(lessee_ctx.to_policy_values(),
                                 self.start_time,
                                 self.end_time_1,
                                 self.max_time)

        assert not mock_authorize.called



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
import threading

from oslo_config import cfg
from oslo_db.sqlalchemy import enginefacade
from oslo_log import log as logging

import sqlalchemy as sa
from sqlalchemy import or_

from esi_leap.common import constants
from esi_leap.common import exception
from esi_leap.common import keystone
from esi_leap.common import statuses
from esi_leap.db.sqlalchemy import models


CONF = cfg.CONF
LOG = logging.getLogger(__name__)

_CONTEXT = threading.local()


def get_backend():
    """The backend is this module itself."""
    return sys.modules[__name__]


def _session_for_read():
    return enginefacade.reader.using(_CONTEXT)


def _session_for_write():
    return enginefacade.writer.using(_CONTEXT)


def model_query(model, *args):
    """Query helper.

    :param model: base model to query
    """
    with _session_for_read() as session:
        query = session.query(model, *args)
        return query


# Helpers for building constraints / equality checks


def constraint(**conditions):
    return Constraint(conditions)


def equal_any(*values):
    return EqualityCondition(values)


def not_equal(*values):
    return InequalityCondition(values)


class Constraint(object):
    def __init__(self, conditions):
        self.conditions = conditions

    def apply(self, model, query):
        for key, condition in self.conditions.items():
            for clause in condition.clauses(getattr(model, key)):
                query = query.filter(clause)
        return query


class EqualityCondition(object):
    def __init__(self, values):
        self.values = values

    def clauses(self, field):
        return sa.or_([field == value for value in self.values])


class InequalityCondition(object):
    def __init__(self, values):
        self.values = values

    def clauses(self, field):
        return [field != value for value in self.values]


# Offer
def offer_get_by_uuid(offer_uuid):
    query = model_query(models.Offer)
    offer_ref = query.filter_by(uuid=offer_uuid).one_or_none()
    return offer_ref


def offer_get_by_name(name):
    query = model_query(models.Offer)
    offers = query.filter_by(name=name).all()
    return offers


def offer_get_all(filters):

    query = model_query(models.Offer)

    lessee_id = filters.pop('lessee_id', None)
    start = filters.pop('start_time', None)
    end = filters.pop('end_time', None)
    time_filter_type = filters.pop('time_filter_type', None)
    a_start = filters.pop('available_start_time', None)
    status = filters.pop('status', None)
    a_end = filters.pop('available_end_time', None)

    query = query.filter_by(**filters)

    if status:
        query = query.filter((models.Offer.status.in_(status)))

    if lessee_id:
        lessee_id_list = keystone.get_parent_project_id_tree(lessee_id)
        query = query.filter(or_(models.Offer.project_id == lessee_id,
                                 models.Offer.lessee_id.__eq__(None),
                                 models.Offer.lessee_id.in_(lessee_id_list)))

    if start and end:
        if time_filter_type == constants.WITHIN_TIME_FILTER:
            query = query.filter(((start <= models.Offer.start_time) &
                                  (end >= models.Offer.start_time)) |

                                 ((start <= models.Offer.end_time) &
                                  (end >= models.Offer.end_time)))
        else:
            query = query.filter((start >= models.Offer.start_time) &
                                 (end <= models.Offer.end_time))

    if a_start and a_end:
        for o in query:
            try:
                offer_verify_availability(o, a_start, a_end)
            except exception.OfferNoTimeAvailabilities:
                query = query.filter(models.Offer.uuid != o.uuid)

    return query


def offer_get_conflict_times(offer_ref):

    l_query = model_query(models.Lease)

    return l_query.with_entities(
        models.Lease.start_time, models.Lease.end_time).\
        join(models.Offer, models.Offer.uuid == models.Lease.offer_uuid).\
        order_by(models.Lease.start_time).\
        filter(models.Lease.offer_uuid == offer_ref.uuid,
               (models.Lease.status != statuses.EXPIRED) &
               (models.Lease.status != statuses.DELETED)
               ).all()


def offer_get_next_lease_start_time(offer_uuid, start):
    l_query = model_query(models.Lease)

    return l_query.with_entities(
        models.Lease.start_time).\
        filter(models.Lease.offer_uuid == offer_uuid,
               (models.Lease.status == statuses.CREATED) |
               (models.Lease.status == statuses.ACTIVE)
               ).\
        order_by(models.Lease.start_time).\
        filter((models.Lease.end_time >= start) &
               (models.Lease.start_time >= start)).first()


def offer_verify_availability(offer_ref, start, end):

    if start < offer_ref.start_time or end > offer_ref.end_time:
        raise exception.OfferNoTimeAvailabilities(offer_uuid=offer_ref.uuid,
                                                  start_time=start,
                                                  end_time=end)

    l_query = model_query(models.Lease)

    leases = l_query.with_entities(
        models.Lease.start_time, models.Lease.end_time).\
        filter((models.Lease.offer_uuid == offer_ref.uuid),
               (models.Lease.status == statuses.CREATED) |
               (models.Lease.status == statuses.ACTIVE)
               )
    leases = add_lease_conflict_filter(leases, start, end)
    conflict = leases.first()

    if conflict:
        raise exception.OfferNoTimeAvailabilities(offer_uuid=offer_ref.uuid,
                                                  start_time=start,
                                                  end_time=end)


def offer_create(values):
    offer_ref = models.Offer()
    offer_ref.update(values)

    with _session_for_write() as session:
        session.add(offer_ref)
        session.flush()
        return offer_ref


def offer_update(offer_uuid, values):

    with _session_for_write() as session:

        query = model_query(models.Offer)
        offer_ref = query.filter_by(uuid=offer_uuid).one_or_none()

        values.pop('uuid', None)
        values.pop('project_id', None)

        start = values.get('start_time', None)
        end = values.get('end_time', None)
        if start is None:
            start = offer_ref.start_time
        if end is None:
            end = offer_ref.end_time
        if start >= end:
            raise exception.InvalidTimeRange(resource='an offer',
                                             start_time=str(start),
                                             end_time=str(end))

        offer_ref.update(values)
        session.flush()
        return offer_ref


def offer_destroy(offer_uuid):
    with _session_for_write() as session:
        query = model_query(models.Offer)
        offer_ref = query.filter_by(uuid=offer_uuid).one_or_none()

        if not offer_ref:
            raise exception.OfferNotFound(offer_uuid=offer_uuid)

        model_query(models.Offer).filter_by(uuid=offer_uuid).delete()
        session.flush()


def add_offer_conflict_filter(query, start, end):
    return query.filter((
        ((start >= models.Offer.start_time) &
         (start < models.Offer.end_time)) |

        ((end > models.Offer.start_time) &
         (end <= models.Offer.end_time)) |

        ((start <= models.Offer.start_time) &
         (end >= models.Offer.end_time))
    ))


# Leases
def lease_get_by_uuid(lease_uuid):
    query = model_query(models.Lease)
    result = query.filter_by(uuid=lease_uuid).one_or_none()
    return result


def lease_get_by_name(name):
    query = model_query(models.Lease)
    leases = query.filter_by(name=name).all()
    return leases


def lease_get_all(filters):
    query = model_query(models.Lease)

    start = filters.pop('start_time', None)
    end = filters.pop('end_time', None)
    time_filter_type = filters.pop('time_filter_type', None)
    status = filters.pop('status', None)
    project_or_owner_id = filters.pop('project_or_owner_id', None)

    query = query.filter_by(**filters)

    if status:
        query = query.filter((models.Lease.status.in_(status)))

    if start and end:
        if time_filter_type == constants.WITHIN_TIME_FILTER:
            query = query.filter(((start <= models.Lease.start_time) &
                                  (end >= models.Lease.start_time)) |

                                 ((start <= models.Lease.end_time) &
                                  (end >= models.Lease.end_time)) |

                                 ((start >= models.Lease.start_time) &
                                  (end <= models.Lease.end_time)))

        else:
            query = query.filter((start >= models.Lease.start_time) &
                                 (end <= models.Lease.end_time))

    if project_or_owner_id:
        query = query.filter(
            (project_or_owner_id == models.Lease.project_id) |
            (project_or_owner_id == models.Lease.owner_id))

    return query


def lease_create(values):
    lease_ref = models.Lease()
    lease_ref.update(values)

    with _session_for_write() as session:
        session.add(lease_ref)
        session.flush()
        return lease_ref


def lease_update(lease_uuid, values):
    with _session_for_write() as session:
        query = model_query(models.Lease)
        lease_ref = query.filter_by(uuid=lease_uuid).one_or_none()

        values.pop('uuid', None)
        values.pop('project_id', None)

        start = values.get('start_time', None)
        end = values.get('end_time', None)
        if start is None:
            start = lease_ref.start_time
        if end is None:
            end = lease_ref.end_time
        if start >= end:
            raise exception.InvalidTimeRange(resource='a lease',
                                             start_time=str(start),
                                             end_time=str(end))

        lease_ref.update(values)
        session.flush()
        return lease_ref


def lease_destroy(lease_uuid):
    with _session_for_write() as session:

        query = model_query(models.Lease)
        lease_ref = query.filter_by(uuid=lease_uuid).one_or_none()

        if not lease_ref:
            raise exception.LeaseNotFound(lease_uuid=lease_uuid)
        query.delete()
        session.flush()


def lease_verify_child_availability(lease_ref, start, end):
    if start < lease_ref.start_time or end > lease_ref.end_time:
        raise exception.LeaseNoTimeAvailabilities(lease_uuid=lease_ref.uuid,
                                                  start_time=start,
                                                  end_time=end)

    # check lease conflicts
    l_query = model_query(models.Lease)
    leases = l_query.with_entities(
        models.Lease.start_time, models.Lease.end_time).\
        filter((models.Lease.parent_lease_uuid == lease_ref.uuid),
               (models.Lease.status == statuses.CREATED) |
               (models.Lease.status == statuses.ACTIVE)
               )
    leases = add_lease_conflict_filter(leases, start, end)
    lease_conflict = leases.first()

    if lease_conflict:
        raise exception.LeaseNoTimeAvailabilities(lease_uuid=lease_ref.uuid,
                                                  start_time=start,
                                                  end_time=end)

    # check offer conflicts
    o_query = model_query(models.Offer)
    offers = o_query.with_entities(
        models.Offer.start_time, models.Offer.end_time).\
        filter((models.Offer.parent_lease_uuid == lease_ref.uuid),
               (models.Offer.status == statuses.AVAILABLE))
    offers = add_offer_conflict_filter(offers, start, end)
    offer_conflict = offers.first()

    if offer_conflict:
        raise exception.LeaseNoTimeAvailabilities(lease_uuid=lease_ref.uuid,
                                                  start_time=start,
                                                  end_time=end)


def add_lease_conflict_filter(query, start, end):
    return query.filter((
        ((start >= models.Lease.start_time) &
         (start < models.Lease.end_time)) |

        ((end > models.Lease.start_time) &
         (end <= models.Lease.end_time)) |

        ((start <= models.Lease.start_time) &
         (end >= models.Lease.end_time))
    ))


# Resources
def resource_verify_availability(r_type, r_uuid, start, end):
    # check conflict with offers
    o_query = model_query(models.Offer)

    offers = o_query.with_entities(
        models.Offer.start_time, models.Offer.end_time).\
        filter((models.Offer.resource_uuid == r_uuid),
               (models.Offer.resource_type == r_type),
               (models.Offer.status == statuses.AVAILABLE))
    offers = add_offer_conflict_filter(offers, start, end)
    offer_conflict = offers.first()

    if offer_conflict:
        raise exception.ResourceTimeConflict(
            resource_uuid=r_uuid,
            resource_type=r_type)

    # check conflict with leases
    l_query = model_query(models.Lease)

    leases = l_query.with_entities(
        models.Lease.start_time, models.Lease.end_time).\
        filter((models.Lease.resource_uuid == r_uuid),
               (models.Lease.resource_type == r_type),
               (models.Lease.status.in_([statuses.CREATED, statuses.ACTIVE])))
    leases = add_lease_conflict_filter(leases, start, end)
    lease_conflict = leases.first()

    if lease_conflict:
        raise exception.ResourceTimeConflict(
            resource_uuid=r_uuid,
            resource_type=r_type)


# Events

def event_get_all(filters):
    query = model_query(models.Event)

    last_event_time = filters.pop('last_event_time', None)
    last_event_id = filters.pop('last_event_id', None)
    lessee_or_owner_id = filters.pop('lessee_or_owner_id', None)

    query = query.filter_by(**filters)

    if last_event_time:
        query = query.filter(
            last_event_time < models.Event.event_time)
    if last_event_id:
        query = query.filter(last_event_id < models.Event.id)
    if lessee_or_owner_id:
        query = query.filter(
            (lessee_or_owner_id == models.Event.lessee_id) |
            (lessee_or_owner_id == models.Event.owner_id))

    return query


def event_create(values):
    event_ref = models.Event()
    event_ref.update(values)

    with _session_for_write() as session:
        session.add(event_ref)
        session.flush()
        return event_ref



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import mock

from esi_leap.common import exception
from esi_leap import resource_objects
from esi_leap.tests import base


class TestResourceObjects(base.TestCase):

    def test_get_type_valid(self):
        types = {'ironic_node': resource_objects.ironic_node.IronicNode,
                 'dummy_node': resource_objects.dummy_node.DummyNode,
                 'test_node': resource_objects.test_node.TestNode}

        for type_name, expected_type in types.items():
            self.assertEqual(resource_objects.get_type(type_name),
                             expected_type)

    def test_get_type_invalid(self):
        invalid_types = ('ahhhh', '', 1234, None, True)
        for type_name in invalid_types:
            self.assertRaises(exception.ResourceTypeUnknown,
                              resource_objects.get_type,
                              type_name)

    @mock.patch('esi_leap.resource_objects.get_type')
    def test_get_resource_object(self, mock_gt):
        mock_type = mock.MagicMock()
        mock_type.return_value = 'i have data!'
        mock_gt.return_value = mock_type
        obj = resource_objects.get_resource_object('fake_node', 'data')

        self.assertEqual(obj, 'i have data!')
        mock_type.assert_called_once_with('data')
        mock_gt.assert_called_once_with('fake_node')

    @mock.patch('esi_leap.resource_objects.ironic_node.is_uuid_like')
    def test_ironic_node(self, mock_iul):
        mock_iul.return_value = True
        node = resource_objects.get_resource_object('ironic_node', '1111')

        mock_iul.assert_called_once_with('1111')
        self.assertIsInstance(node, resource_objects.ironic_node.IronicNode)
        self.assertEqual('1111', node.get_uuid())

    @mock.patch('esi_leap.resource_objects.ironic_node.get_ironic_client')
    @mock.patch('esi_leap.resource_objects.ironic_node.is_uuid_like')
    def test_ironic_node_by_name(self, mock_iul, mock_gic):
        mock_ironic_client = mock.MagicMock()
        mock_ironic_node = mock.MagicMock()
        mock_uuid = mock.PropertyMock()
        mock_uuid.return_value = '1111'
        type(mock_ironic_node).uuid = mock_uuid
        mock_ironic_client.node.get.return_value = mock_ironic_node
        mock_gic.return_value = mock_ironic_client
        mock_iul.return_value = False
        node = resource_objects.get_resource_object('ironic_node', 'node-name')

        mock_iul.assert_called_with('node-name')
        mock_gic.assert_called_once_with()
        mock_uuid.assert_called_once_with()
        mock_ironic_client.node.get.assert_called_once_with('node-name')
        self.assertIsInstance(node, resource_objects.ironic_node.IronicNode)
        self.assertEqual('1111', node.get_uuid())

    def test_dummy_node(self):
        node = resource_objects.get_resource_object('dummy_node', '1111')

        self.assertIsInstance(node, resource_objects.dummy_node.DummyNode)
        self.assertEqual('1111', node.get_uuid())

    def test_test_node(self):
        node = resource_objects.get_resource_object('test_node', '1111')

        self.assertIsInstance(node, resource_objects.test_node.TestNode)
        self.assertEqual('1111', node.get_uuid())

    def test_unknown_resource_type(self):
        self.assertRaises(exception.ResourceTypeUnknown,
                          resource_objects.get_resource_object,
                          'foo_node', '1111')



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import oslo_messaging as messaging

import esi_leap.conf
from esi_leap.manager import utils

CONF = esi_leap.conf.CONF


class ManagerRPCAPI(object):
    """Client side of the manager RPC API

    API version history:

    * 1.0 - Initial version.
    """

    def __init__(self):
        self._client = messaging.RPCClient(
            target=utils.get_target(),
            transport=messaging.get_rpc_transport(CONF))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""create events table

Revision ID: a1ea63fec697
Revises:
Create Date: 2023-06-26 14:22:34.822066

"""
from alembic import op
import sqlalchemy as sa


# revision identifiers, used by Alembic.
revision = 'a1ea63fec697'
down_revision = None
branch_labels = None
depends_on = None


def upgrade():
    op.create_table(
        'events',
        sa.Column('id', sa.Integer(), nullable=False),
        sa.Column('event_type', sa.String(length=255), nullable=False),
        sa.Column('event_time', sa.DateTime(), nullable=False),
        sa.Column('object_type', sa.String(length=255), nullable=True),
        sa.Column('object_uuid', sa.String(length=36), nullable=True),
        sa.Column('resource_type', sa.String(length=255), nullable=True),
        sa.Column('resource_uuid', sa.String(length=36), nullable=True),
        sa.Column('lessee_id', sa.String(length=36), nullable=True),
        sa.Column('owner_id', sa.String(length=36), nullable=True),
        sa.Column('created_at', sa.DateTime(), nullable=True),
        sa.Column('updated_at', sa.DateTime(), nullable=True),
        sa.PrimaryKeyConstraint('id'),
    )

    op.create_index('event_type_idx', 'events', ['event_type'],
                    unique=False)
    op.create_index('event_lessee_id_idx', 'events', ['lessee_id'],
                    unique=False)
    op.create_index('event_owner_id_idx', 'events', ['owner_id'],
                    unique=False)
    op.create_index('event_resource_idx', 'events',
                    ['resource_type', 'resource_uuid'],
                    unique=False)


def downgrade():
    pass



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import concurrent.futures
from datetime import datetime
import pecan
from pecan import rest
import wsme
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from esi_leap.api.controllers import base
from esi_leap.api.controllers import types
from esi_leap.common import ironic
from esi_leap.common import keystone
from esi_leap.common import statuses
import esi_leap.conf
from esi_leap.objects import lease as lease_obj
from esi_leap.objects import offer as offer_obj

CONF = esi_leap.conf.CONF


class Node(base.ESILEAPBase):

    name = wsme.wsattr(wtypes.text)
    owner = wsme.wsattr(wtypes.text)
    maintenance = wsme.wsattr(wtypes.text)
    provision_state = wsme.wsattr(wtypes.text)
    uuid = wsme.wsattr(wtypes.text)
    offer_uuid = wsme.wsattr(wtypes.text)
    lease_uuid = wsme.wsattr(wtypes.text)
    lessee = wsme.wsattr(wtypes.text)
    future_offers = wsme.wsattr(wtypes.text)
    future_leases = wsme.wsattr(wtypes.text)

    def __init__(self, **kwargs):
        self.fields = ('name', 'owner', 'uuid', 'offer_uuid', 'lease_uuid',
                       'lessee', 'future_offers', 'future_leases',
                       'provision_state', 'maintenance')
        for field in self.fields:
            setattr(self, field, kwargs.get(field, wtypes.Unset))


class NodeCollection(types.Collection):
    nodes = [Node]

    def __init__(self, **kwargs):
        self._type = 'nodes'
        self.nodes = kwargs.get('nodes', [])


class NodesController(rest.RestController):

    @wsme_pecan.wsexpose(NodeCollection)
    def get_all(self):
        context = pecan.request.context

        nodes = None
        project_list = None

        with concurrent.futures.ThreadPoolExecutor() as executor:
            f1 = executor.submit(ironic.get_node_list, context)
            f2 = executor.submit(keystone.get_project_list)
            nodes = f1.result()
            project_list = f2.result()

        node_collection = NodeCollection()

        now = datetime.now()

        offers = offer_obj.Offer.get_all({'status': [statuses.AVAILABLE]},
                                         context)

        leases = lease_obj.Lease.get_all({'status': [statuses.CREATED]},
                                         context)

        for node in nodes:
            future_offers = []
            current_offer = None

            node_offers = [offer for offer in offers
                           if offer.resource_uuid == node.uuid]

            for offer in node_offers:
                if offer.start_time > now:
                    future_offers.append(offer.uuid)
                elif offer.end_time >= now:
                    current_offer = offer
            future_offers = ' '.join(future_offers)

            f_lease_uuids = ''.join([lease.uuid for lease in leases
                                     if lease.resource_uuid == node.uuid])

            n = Node(name=node.name, uuid=node.uuid,
                     provision_state=node.provision_state,
                     maintenance=str(node.maintenance),
                     owner=keystone.get_project_name(node.owner, project_list),
                     lessee=keystone.get_project_name(node.lessee,
                                                      project_list),
                     future_offers=future_offers,
                     future_leases=f_lease_uuids)

            if current_offer:
                n.offer_uuid = current_offer.uuid
            if 'lease_uuid' in node.properties:
                n.lease_uuid = node.properties['lease_uuid']

            node_collection.nodes.append(n)

        return node_collection



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from oslo_service import service

from esi_leap.common import service as esi_leap_service
import esi_leap.conf
from esi_leap.manager import service as manager_service


CONF = esi_leap.conf.CONF


def main():
    esi_leap_service.prepare_service(sys.argv)
    service.launch(
        CONF,
        manager_service.ManagerService(),
        restart_method='mutate'
    ).wait()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from datetime import datetime
import mock

from esi_leap.common import exception
from esi_leap.resource_objects.test_node import TestNode
from esi_leap.tests.api import base as test_api_base


class FakeEvent(object):
    def __init__(self):
        self.id = 1
        self.event_type = 'fake:event'
        self.event_time = datetime.now()
        self.object_type = 'lease'
        self.object_uuid = 'fake-lease-uuid'
        self.resource_type = 'fake_node'
        self.resource_uuid = 'fake-node-uuid'
        self.lessee_id = 'fake-lessee-id'
        self.owner_id = 'fake-owner-id'


class TestEventsController(test_api_base.APITestCase):

    def setUp(self):
        super(TestEventsController, self).setUp()

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_project_uuid_from_ident')
    @mock.patch('esi_leap.api.controllers.v1.event.get_resource_object')
    @mock.patch('esi_leap.objects.event.Event.get_all')
    def test_get_all(self, mock_ega, mock_gro, mock_gpufi, mock_pa):
        fake_event = FakeEvent()
        expected_filters = {}
        mock_pa.side_effect = None
        mock_ega.return_value = [fake_event]

        data = self.get_json('/events')

        mock_pa.assert_called_once()
        mock_gpufi.assert_not_called()
        mock_gro.assert_not_called()
        mock_ega.assert_called_once_with(expected_filters, self.context)

        self.assertEqual(data['events'][0]['id'], 1)

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_project_uuid_from_ident')
    @mock.patch('esi_leap.api.controllers.v1.event.get_resource_object')
    @mock.patch('esi_leap.objects.event.Event.get_all')
    def test_get_all_not_admin(self, mock_ega, mock_gro, mock_gpufi, mock_pa):
        fake_event = FakeEvent()
        expected_filters = {'lessee_or_owner_id': 'fake-lessee-id'}
        mock_pa.side_effect = exception.HTTPForbidden(
            rule='esi_leap:offer:offer_admin')
        mock_gpufi.return_value = 'fake-lessee-id'
        mock_ega.return_value = [fake_event]

        data = self.get_json('/events')

        mock_pa.assert_called_once()
        mock_gpufi.assert_called_once()
        mock_gro.assert_not_called()
        mock_ega.assert_called_once_with(expected_filters, self.context)

        self.assertEqual(data['events'][0]['id'], 1)

    @mock.patch('esi_leap.api.controllers.v1.utils.policy_authorize')
    @mock.patch('esi_leap.common.keystone.get_project_uuid_from_ident')
    @mock.patch('esi_leap.api.controllers.v1.event.get_resource_object')
    @mock.patch('esi_leap.objects.event.Event.get_all')
    def test_get_all_resource_filter(self, mock_ega, mock_gro, mock_gpufi,
                                     mock_pa):
        fake_event = FakeEvent()
        expected_filters = {'resource_type': 'test_node',
                            'resource_uuid': '1111'}
        mock_pa.side_effect = None
        mock_gro.return_value = TestNode('1111')
        mock_ega.return_value = [fake_event]

        data = self.get_json(
            '/events?resource_uuid=1111&resource_type=test_node')

        mock_pa.assert_called_once()
        mock_gpufi.assert_not_called()
        mock_gro.assert_called_with('test_node', '1111')
        mock_ega.assert_called_once_with(expected_filters, self.context)

        self.assertEqual(data['events'][0]['id'], 1)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from keystoneauth1 import loading as ks_loading
from keystoneauth1 import service_token
from keystoneauth1 import token_endpoint

from ironicclient import client as ironic_client

import esi_leap.conf


CONF = esi_leap.conf.CONF
_cached_ironic_client = None


def get_ironic_client(context=None):
    session = ks_loading.load_session_from_conf_options(CONF, 'ironic')
    service_auth = ks_loading.load_auth_from_conf_options(CONF, 'ironic')

    # use user context if provided
    user_auth = None
    if context:
        endpoint = ks_loading.load_adapter_from_conf_options(
            CONF, 'ironic', session=session, auth=service_auth).get_endpoint()
        user_auth = service_token.ServiceTokenAuthWrapper(
            user_auth=token_endpoint.Token(endpoint, context.auth_token),
            service_auth=service_auth)
    sess = ks_loading.load_session_from_conf_options(
        CONF, 'ironic', auth=user_auth or service_auth)

    kwargs = {'os_ironic_api_version': '1.65'}
    cli = ironic_client.get_client(1, session=sess, **kwargs)
    return cli


def get_node_list(context=None):
    return get_ironic_client(context).node.list(detail=True)


def get_node(node_uuid, node_list=None):
    if node_list is None:
        node = get_ironic_client().node.get(node_uuid)
    else:
        node = next((n for n in node_list if n.uuid == node_uuid), None)
    return node



import setuptools

# In python < 2.7.4, a lazy loading of package `pbr` will break
# setuptools if some other modules registered functions in `atexit`.
# solution from: http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing  # noqa pylint: disable=unused-import
except ImportError:
    pass

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text()

setuptools.setup(
    setup_requires=['pbr>=2.0.0'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    pbr=True)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import mock
from oslo_utils import uuidutils

from esi_leap.common import statuses
from esi_leap.manager.service import ManagerService
from esi_leap.objects import lease
from esi_leap.objects import offer
from esi_leap.tests import base


class TestService(base.TestCase):

    def setUp(self):
        super(TestService, self).setUp()

        self.test_offer = offer.Offer(
            resource_type='test_node',
            resource_uuid='abc',
            name='o',
            uuid=uuidutils.generate_uuid(),
            status=statuses.AVAILABLE,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
            project_id='ownerid'
        )

        self.test_lease = lease.Lease(
            offer_uuid=self.test_offer.uuid,
            name='c',
            uuid=uuidutils.generate_uuid(),
            project_id='lesseeid',
            status=statuses.CREATED,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
        )

    @mock.patch('esi_leap.objects.lease.Lease.fulfill')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__fulfill_leases(self, mock_ga, mock_utcnow, mock_fulfill):
        mock_ga.return_value = [self.test_lease, self.test_lease]
        mock_utcnow.return_value = datetime.datetime(3500, 7, 16)

        s = ManagerService()
        s._fulfill_leases()

        assert mock_fulfill.call_count == 2
        mock_ga.assert_called_once_with({
            'status': [statuses.CREATED, statuses.WAIT_FULFILL]
        }, s._context)

    @mock.patch('esi_leap.objects.lease.Lease.save')
    @mock.patch('esi_leap.objects.lease.Lease.fulfill')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__fulfill_leases_error(self, mock_ga, mock_utcnow, mock_fulfill,
                                   mock_save):
        error_lease = lease.Lease(
            offer_uuid=self.test_offer.uuid,
            name='c',
            uuid=uuidutils.generate_uuid(),
            project_id='lesseeid',
            status=statuses.CREATED,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
        )
        mock_ga.return_value = [error_lease]
        mock_utcnow.return_value = datetime.datetime(3500, 7, 16)
        mock_fulfill.side_effect = Exception('whoops')

        s = ManagerService()
        s._fulfill_leases()

        mock_fulfill.assert_called_once()
        mock_ga.assert_called_once_with({
            'status': [statuses.CREATED, statuses.WAIT_FULFILL]
        }, s._context)
        self.assertEqual(statuses.ERROR, error_lease.status)
        mock_save.assert_called_once()

    @mock.patch('esi_leap.objects.lease.Lease.expire')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__expire_leases(self, mock_ga, mock_utcnow, mock_expire):
        mock_ga.return_value = [self.test_lease, self.test_lease]
        mock_utcnow.return_value = datetime.datetime(5000, 7, 16)

        s = ManagerService()
        s._expire_leases()

        assert mock_expire.call_count == 2
        mock_ga.assert_called_once_with({
            'status': [statuses.ACTIVE, statuses.CREATED,
                       statuses.WAIT_EXPIRE, statuses.WAIT_FULFILL]
        }, s._context)

    @mock.patch('esi_leap.objects.lease.Lease.save')
    @mock.patch('esi_leap.objects.lease.Lease.expire')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__expire_leases_error(self, mock_ga, mock_utcnow, mock_expire,
                                  mock_save):
        error_lease = lease.Lease(
            offer_uuid=self.test_offer.uuid,
            name='c',
            uuid=uuidutils.generate_uuid(),
            project_id='lesseeid',
            status=statuses.CREATED,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
        )
        mock_ga.return_value = [error_lease]
        mock_utcnow.return_value = datetime.datetime(5000, 7, 16)
        mock_expire.side_effect = Exception('whoops')

        s = ManagerService()
        s._expire_leases()

        mock_expire.assert_called_once()
        mock_ga.assert_called_once_with({
            'status': [statuses.ACTIVE, statuses.CREATED,
                       statuses.WAIT_EXPIRE, statuses.WAIT_FULFILL]
        }, s._context)
        self.assertEqual(statuses.ERROR, error_lease.status)
        mock_save.assert_called_once()

    @mock.patch('esi_leap.objects.lease.Lease.cancel')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__cancel_leases(self, mock_ga, mock_utcnow, mock_cancel):
        mock_ga.return_value = [self.test_lease, self.test_lease]

        s = ManagerService()
        s._cancel_leases()

        assert mock_cancel.call_count == 2
        mock_ga.assert_called_once_with({
            'status': [statuses.WAIT_CANCEL]
        }, s._context)

    @mock.patch('esi_leap.objects.lease.Lease.save')
    @mock.patch('esi_leap.objects.lease.Lease.cancel')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.lease.Lease.get_all')
    def test__cancel_leases_error(self, mock_ga, mock_utcnow, mock_cancel,
                                  mock_save):
        error_lease = lease.Lease(
            offer_uuid=self.test_offer.uuid,
            name='c',
            uuid=uuidutils.generate_uuid(),
            project_id='lesseeid',
            status=statuses.WAIT_CANCEL,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
        )
        mock_ga.return_value = [error_lease]
        mock_utcnow.return_value = datetime.datetime(5000, 7, 16)
        mock_cancel.side_effect = Exception('whoops')

        s = ManagerService()
        s._cancel_leases()

        mock_cancel.assert_called_once()
        mock_ga.assert_called_once_with({
            'status': [statuses.WAIT_CANCEL]
        }, s._context)
        self.assertEqual(statuses.ERROR, error_lease.status)
        mock_save.assert_called_once()

    @mock.patch('esi_leap.objects.offer.Offer.expire')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.offer.Offer.get_all')
    def test__expire_offers(self, mock_ga, mock_utcnow, mock_expire):
        mock_ga.return_value = [self.test_offer, self.test_offer]
        mock_utcnow.return_value = datetime.datetime(5000, 7, 16)

        s = ManagerService()
        s._expire_offers()

        assert mock_expire.call_count == 2
        mock_ga.assert_called_once_with({
            'status': statuses.OFFER_CAN_DELETE
        }, s._context)

    @mock.patch('esi_leap.objects.offer.Offer.save')
    @mock.patch('esi_leap.objects.offer.Offer.expire')
    @mock.patch('oslo_utils.timeutils.utcnow')
    @mock.patch('esi_leap.objects.offer.Offer.get_all')
    def test__expire_offers_error(self, mock_ga, mock_utcnow, mock_expire,
                                  mock_save):
        error_offer = offer.Offer(
            resource_type='test_node',
            resource_uuid='abc',
            name='o',
            uuid=uuidutils.generate_uuid(),
            status=statuses.AVAILABLE,
            start_time=datetime.datetime(3000, 7, 16),
            end_time=datetime.datetime(4000, 7, 16),
            project_id='ownerid'
        )
        mock_ga.return_value = [error_offer]
        mock_utcnow.return_value = datetime.datetime(5000, 7, 16)
        mock_expire.side_effect = Exception('whoops')

        s = ManagerService()
        s._expire_offers()

        mock_expire.assert_called_once()
        mock_ga.assert_called_once_with({
            'status': statuses.OFFER_CAN_DELETE
        }, s._context)
        self.assertEqual(statuses.ERROR, error_offer.status)
        mock_save.assert_called_once()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import datetime
import pecan
from pecan import rest
import wsme
from wsme import types as wtypes
import wsmeext.pecan as wsme_pecan

from esi_leap.api.controllers import base
from esi_leap.api.controllers import types
from esi_leap.api.controllers.v1 import utils
from esi_leap.common import exception
from esi_leap.common import keystone
import esi_leap.conf
from esi_leap.objects import event as event_obj
from esi_leap.resource_objects import get_resource_object

CONF = esi_leap.conf.CONF


class Event(base.ESILEAPBase):

    id = wsme.wsattr(int, readonly=True)
    event_type = wsme.wsattr(wtypes.text, readonly=True)
    event_time = wsme.wsattr(datetime.datetime, readonly=True)
    object_type = wsme.wsattr(wtypes.text, readonly=True)
    object_uuid = wsme.wsattr(wtypes.text, readonly=True)
    resource_type = wsme.wsattr(wtypes.text, readonly=True)
    resource_uuid = wsme.wsattr(wtypes.text, readonly=True)
    lessee_id = wsme.wsattr(wtypes.text, readonly=True)
    owner_id = wsme.wsattr(wtypes.text, readonly=True)

    def __init__(self, **kwargs):

        self.fields = event_obj.Event.fields
        for field in self.fields:
            setattr(self, field, kwargs.get(field, wtypes.Unset))


class EventCollection(types.Collection):
    events = [Event]

    def __init__(self, **kwargs):
        self._type = 'events'


class EventsController(rest.RestController):

    @wsme_pecan.wsexpose(EventCollection, int, wtypes.text,
                         datetime.datetime, wtypes.text, wtypes.text,
                         wtypes.text, wtypes.text)
    def get_all(self, last_event_id=None, lessee_or_owner_id=None,
                last_event_time=None, event_type=None,
                resource_type=None, resource_uuid=None):
        request = pecan.request.context
        cdict = request.to_policy_values()

        try:
            utils.policy_authorize('esi_leap:offer:offer_admin', cdict, cdict)
        except exception.HTTPForbidden:
            lessee_or_owner_id = cdict['project_id']

        if lessee_or_owner_id is not None:
            lessee_or_owner_id = keystone.get_project_uuid_from_ident(
                lessee_or_owner_id)

        if resource_uuid is not None:
            if resource_type is None:
                resource_type = CONF.api.default_resource_type
            resource = get_resource_object(resource_type, resource_uuid)
            resource_uuid = resource.get_uuid()

        filters = {
            'last_event_id': last_event_id,
            'last_event_time': last_event_time,
            'lessee_or_owner_id': lessee_or_owner_id,
            'event_type': event_type,
            'resource_type': resource_type,
            'resource_uuid': resource_uuid,
        }

        # unpack iterator to tuple so we can use 'del'
        for k, v in tuple(filters.items()):
            if v is None:
                del filters[k]

        events = event_obj.Event.get_all(filters, request)
        event_collection = EventCollection()
        event_collection.events = []
        for event in events:
            e = Event(id=event.id,
                      event_type=event.event_type,
                      event_time=event.event_time,
                      object_type=event.object_type,
                      object_uuid=event.object_uuid,
                      resource_type=event.resource_type,
                      resource_uuid=event.resource_uuid,
                      lessee_id=event.lessee_id,
                      owner_id=event.owner_id)
            event_collection.events.append(e)

        return event_collection



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from __future__ import print_function

import sys

from oslo_config import cfg

from esi_leap.common.i18n import _
from esi_leap.common import service
import esi_leap.conf
from esi_leap.db import migration


CONF = esi_leap.conf.CONF


class DBCommand(object):

    def create_schema(self):
        migration.create_schema()

    def upgrade(self):
        migration.upgrade(CONF.command.revision)

    def downgrade(self):
        migration.downgrade(CONF.command.revision)

    def stamp(self):
        migration.stamp(CONF.command.revision)

    def revision(self):
        migration.revision(CONF.command.message, CONF.command.autogenerate)

    def version(self):
        print(migration.version())


def add_command_parsers(subparsers):
    command_object = DBCommand()

    parser = subparsers.add_parser(
        'create_schema',
        help=_('Create the database schema.'))
    parser.set_defaults(func=command_object.create_schema)

    parser = subparsers.add_parser(
        'upgrade',
        help=_('Upgrade the database.'))
    parser.set_defaults(func=command_object.upgrade)
    parser.add_argument('--revision', nargs='?')

    parser = subparsers.add_parser(
        'downgrade',
        help=_('Downgrade the database.'))
    parser.set_defaults(func=command_object.downgrade)
    parser.add_argument('--revision', nargs='?')

    parser = subparsers.add_parser(
        'stamp',
        help=_('Stamp the database with provided revision.'))
    parser.set_defaults(func=command_object.stamp)
    parser.add_argument('--revision', nargs='?')

    parser = subparsers.add_parser(
        'revision',
        help=_('Creates template for migration'))
    parser.set_defaults(func=command_object.revision)
    parser.add_argument('-m', '--message')
    parser.add_argument('--autogenerate', action='store_true')

    parser = subparsers.add_parser(
        'version',
        help=_('Print the current version information and exit.'))
    parser.set_defaults(func=command_object.version)


def main():
    command_opt = cfg.SubCommandOpt('command',
                                    title='Command',
                                    help=_('Available commands'),
                                    handler=add_command_parsers)

    CONF.register_cli_opt(command_opt)

    service.prepare_service(sys.argv)
    CONF.command.func()



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import fixtures

from oslo_concurrency import lockutils
from oslo_config import fixture as config
from oslo_context import context as ctx
from oslo_db.sqlalchemy import enginefacade
from oslotest import base

import esi_leap.conf
from esi_leap.db import api as db_api
from esi_leap.db.sqlalchemy import models


_DB_CACHE = None
CONF = esi_leap.conf.CONF


class Database(fixtures.Fixture):

    def __init__(self, engine, sql_connection, sqlite_clean_db):
        self.sql_connection = sql_connection
        self.sqlite_clean_db = sqlite_clean_db

        self.engine = engine
        self.engine.dispose()

        models.Base.metadata.create_all(self.engine)

        conn = self.engine.connect()
        self._DB = ''.join(line for line in conn.connection.iterdump())
        self.engine.dispose()

    def setUp(self):
        super(Database, self).setUp()

        if self.sql_connection == 'sqlite://':
            conn = self.engine.connect()
            conn.connection.executescript(self._DB)
            self.addCleanup(self.engine.dispose)


class TestCase(base.BaseTestCase):

    def setUp(self):
        self.config = self.useFixture(config.Config(lockutils.CONF)).config
        super(TestCase, self).setUp()

        if not hasattr(self, 'context'):
            self.context = ctx.RequestContext(
                auth_token=None,
                project_id='12345',
                is_admin=True,
                overwrite=False)


class DBTestCase(TestCase):

    def setUp(self):
        super(DBTestCase, self).setUp()
        CONF.set_override('connection', 'sqlite://', group='database')

        self.db_api = db_api.get_instance()

        global _DB_CACHE
        if not _DB_CACHE:
            engine = enginefacade.get_legacy_facade().get_engine()
            _DB_CACHE = Database(engine,
                                 sql_connection=CONF.database.connection,
                                 sqlite_clean_db='clean.sqlite')
        self.useFixture(_DB_CACHE)
